DIRECTIONAL = {
    "north": "N",
    "northeast": "NE",
    "east": "E",
    "southeast": "SE",
    "south": "S",
    "southwest": "SW",
    "west": "W",
    "northwest": "NW",
}

DIRECTION_CODE = {
    "N": "north",
    "NE": "northeast",
    "E": "east",
    "SE": "southeast",
    "SW": "southwest",
    "W": "west",
    "NW": "northwest",
}

STREET_TYPE = {
    "allee": "aly",
    "alley": "aly",
    "ally": "aly",
    "anex": "anx",
    "annex": "anx",
    "annx": "anx",
    "arcade": "arc",
    "av": "ave",
    "aven": "ave",
    "avenu": "ave",
    "avenue": "ave",
    "avn": "ave",
    "avnue": "ave",
    "bayoo": "byu",
    "bayou": "byu",
    "beach": "bch",
    "bend": "bnd",
    "bluf": "blf",
    "bluff": "blf",
    "bluffs": "blfs",
    "bot": "btm",
    "bottm": "btm",
    "bottom": "btm",
    "boul": "blvd",
    "boulevard": "blvd",
    "boulv": "blvd",
    "branch": "br",
    "brdge": "brg",
    "bridge": "brg",
    "brnch": "br",
    "brook": "brk",
    "brooks": "brks",
    "burg": "bg",
    "burgs": "bgs",
    "bypa": "byp",
    "bypas": "byp",
    "bypass": "byp",
    "byps": "byp",
    "camp": "cp",
    "canyn": "cyn",
    "canyon": "cyn",
    "cape": "cpe",
    "causeway": "cswy",
    "causway": "cswy",
    "causwa": "cswy",
    "cen": "ctr",
    "cent": "ctr",
    "center": "ctr",
    "centers": "ctrs",
    "centr": "ctr",
    "centre": "ctr",
    "circ": "cir",
    "circl": "cir",
    "circle": "cir",
    "circles": "cirs",
    "ck": "crk",
    "cliff": "clf",
    "cliffs": "clfs",
    "club": "clb",
    "cmp": "cp",
    "cnter": "ctr",
    "cntr": "ctr",
    "cnyn": "cyn",
    "common": "cmn",
    "commons": "cmns",
    "corner": "cor",
    "corners": "cors",
    "course": "crse",
    "court": "ct",
    "courts": "cts",
    "cove": "cv",
    "coves": "cvs",
    "cr": "crk",
    "crcl": "cir",
    "crcle": "cir",
    "crecent": "cres",
    "creek": "crk",
    "crescent": "cres",
    "cresent": "cres",
    "crest": "crst",
    "crossing": "xing",
    "crossroad": "xrd",
    "crossroads": "xrds",
    "crscnt": "cres",
    "crsent": "cres",
    "crsnt": "cres",
    "crssing": "xing",
    "crssng": "xing",
    "crt": "ct",
    "curve": "curv",
    "dale": "dl",
    "dam": "dm",
    "div": "dv",
    "divide": "dv",
    "driv": "dr",
    "drive": "dr",
    "drives": "drs",
    "drv": "dr",
    "dvd": "dv",
    "estate": "est",
    "estates": "ests",
    "exp": "expy",
    "expr": "expy",
    "express": "expy",
    "expressway": "expy",
    "expw": "expy",
    "extension": "ext",
    "extensions": "exts",
    "extn": "ext",
    "extnsn": "ext",
    "fall": "fall",
    "falls": "fls",
    "ferry": "fry",
    "field": "fld",
    "fields": "flds",
    "flat": "flt",
    "flats": "flts",
    "ford": "frd",
    "fords": "frds",
    "forest": "frst",
    "forests": "frst",
    "forg": "frg",
    "forge": "frg",
    "forges": "frgs",
    "fork": "frk",
    "forks": "frks",
    "fort": "ft",
    "freeway": "fwy",
    "freewy": "fwy",
    "frry": "fry",
    "frt": "ft",
    "frway": "fwy",
    "frwy": "fwy",
    "garden": "gdn",
    "gardens": "gdns",
    "gardn": "gdn",
    "gateway": "gtwy",
    "gatewy": "gtwy",
    "gatway": "gtwy",
    "glen": "gln",
    "glens": "glns",
    "grden": "gdn",
    "grdn": "gdn",
    "grdns": "gdns",
    "green": "grn",
    "greens": "grns",
    "grov": "grv",
    "grove": "grv",
    "groves": "grvs",
    "gtway": "gtwy",
    "harb": "hbr",
    "harbor": "hbr",
    "harbors": "hbrs",
    "harbr": "hbr",
    "haven": "hvn",
    "havn": "hvn",
    "height": "hts",
    "heights": "hts",
    "hgts": "hts",
    "highway": "hwy",
    "highwy": "hwy",
    "hill": "hl",
    "hills": "hls",
    "hiway": "hwy",
    "hiwy": "hwy",
    "hllw": "holw",
    "hollow": "holw",
    "hollows": "holw",
    "holws": "holw",
    "hrbor": "hbr",
    "ht": "hts",
    "hway": "hwy",
    "inlet": "inlt",
    "island": "is",
    "islands": "iss",
    "isles": "isle",
    "islnd": "is",
    "islnds": "iss",
    "jction": "jct",
    "jctn": "jct",
    "jctns": "jcts",
    "junction": "jct",
    "junctions": "jcts",
    "junctn": "jct",
    "juncton": "jct",
    "key": "ky",
    "keys": "kys",
    "knol": "knl",
    "knoll": "knl",
    "knolls": "knls",
    "la": "ln",
    "lake": "lk",
    "lakes": "lks",
    "land": "land",
    "landing": "lndg",
    "lane": "ln",
    "lanes": "ln",
    "ldge": "ldg",
    "light": "lgt",
    "lights": "lgts",
    "lndng": "lndg",
    "loaf": "lf",
    "lock": "lck",
    "locks": "lcks",
    "lodg": "ldg",
    "lodge": "ldg",
    "loops": "loop",
    "mall": "mall",
    "manor": "mnr",
    "manors": "mnrs",
    "meadow": "mdw",
    "meadows": "mdws",
    "medows": "mdws",
    "mews": "mews",
    "mill": "ml",
    "mills": "mls",
    "mission": "msn",
    "missn": "msn",
    "mnt": "mt",
    "mntain": "mtn",
    "mntn": "mtn",
    "mntns": "mtns",
    "motorway": "mtwy",
    "mount": "mt",
    "mountain": "mtn",
    "mountains": "mtns",
    "mountin": "mtn",
    "mssn": "msn",
    "mtin": "mtn",
    "neck": "nck",
    "orchard": "orch",
    "orchrd": "orch",
    "overpass": "opas",
    "ovl": "oval",
    "parks": "park",
    "parkway": "pkwy",
    "parkways": "pkwy",
    "parkwy": "pkwy",
    "pass": "pass",
    "passage": "psge",
    "paths": "path",
    "pikes": "pike",
    "pine": "pne",
    "pines": "pnes",
    "pk": "park",
    "pkway": "pkwy",
    "pkwys": "pkwy",
    "pky": "pkwy",
    "place": "pl",
    "plain": "pln",
    "plaines": "plns",
    "plains": "plns",
    "plaza": "plz",
    "plza": "plz",
    "point": "pt",
    "points": "pts",
    "port": "prt",
    "ports": "prts",
    "prairie": "pr",
    "prarie": "pr",
    "prk": "park",
    "prr": "pr",
    "rad": "radl",
    "radial": "radl",
    "radiel": "radl",
    "ranch": "rnch",
    "ranches": "rnch",
    "rapid": "rpd",
    "rapids": "rpds",
    "rdge": "rdg",
    "rest": "rst",
    "ridge": "rdg",
    "ridges": "rdgs",
    "river": "riv",
    "rivr": "riv",
    "rnchs": "rnch",
    "road": "rd",
    "roads": "rds",
    "route": "rte",
    "rvr": "riv",
    "row": "row",
    "rue": "rue",
    "run": "run",
    "shoal": "shl",
    "shoals": "shls",
    "shoar": "shr",
    "shoars": "shrs",
    "shore": "shr",
    "shores": "shrs",
    "skyway": "skwy",
    "spng": "spg",
    "spngs": "spgs",
    "spring": "spg",
    "springs": "spgs",
    "sprng": "spg",
    "sprngs": "spgs",
    "spurs": "spur",
    "sqr": "sq",
    "sqre": "sq",
    "sqrs": "sqs",
    "squ": "sq",
    "square": "sq",
    "squares": "sqs",
    "station": "sta",
    "statn": "sta",
    "stn": "sta",
    "str": "st",
    "strav": "stra",
    "strave": "stra",
    "straven": "stra",
    "stravenue": "stra",
    "stravn": "stra",
    "stream": "strm",
    "street": "st",
    "streets": "sts",
    "streme": "strm",
    "strt": "st",
    "strvn": "stra",
    "strvnue": "stra",
    "sumit": "smt",
    "sumitt": "smt",
    "summit": "smt",
    "terr": "ter",
    "terrace": "ter",
    "throughway": "trwy",
    "tpk": "tpke",
    "tr": "trl",
    "trace": "trce",
    "traces": "trce",
    "track": "trak",
    "tracks": "trak",
    "trafficway": "trfy",
    "trail": "trl",
    "trails": "trl",
    "trk": "trak",
    "trks": "trak",
    "trls": "trl",
    "trnpk": "tpke",
    "trpk": "tpke",
    "tunel": "tunl",
    "tunls": "tunl",
    "tunnel": "tunl",
    "tunnels": "tunl",
    "tunnl": "tunl",
    "turnpike": "tpke",
    "turnpk": "tpke",
    "underpass": "upas",
    "union": "un",
    "unions": "uns",
    "valley": "vly",
    "valleys": "vlys",
    "vally": "vly",
    "vdct": "via",
    "viadct": "via",
    "viaduct": "via",
    "view": "vw",
    "views": "vws",
    "vill": "vlg",
    "villag": "vlg",
    "village": "vlg",
    "villages": "vlgs",
    "ville": "vl",
    "villg": "vlg",
    "villiage": "vlg",
    "vist": "vis",
    "vista": "vis",
    "vlly": "vly",
    "vst": "vis",
    "vsta": "vis",
    "wall": "wall",
    "walks": "walk",
    "well": "wl",
    "wells": "wls",
    "wy": "way",
}

STATE_CODE = {
    "alabama": "AL",
    "alaska": "AK",
    "american samoa": "AS",
    "arizona": "AZ",
    "arkansas": "AR",
    "california": "CA",
    "colorado": "CO",
    "connecticut": "CT",
    "delaware": "DE",
    "district of columbia": "DC",
    "federated states of micronesia": "FM",
    "florida": "FL",
    "georgia": "GA",
    "guam": "GU",
    "hawaii": "HI",
    "idaho": "ID",
    "illinois": "IL",
    "indiana": "IN",
    "iowa": "IA",
    "kansas": "KS",
    "kentucky": "KY",
    "louisiana": "LA",
    "maine": "ME",
    "marshall islands": "MH",
    "maryland": "MD",
    "massachusetts": "MA",
    "michigan": "MI",
    "minnesota": "MN",
    "mississippi": "MS",
    "missouri": "MO",
    "montana": "MT",
    "nebraska": "NE",
    "nevada": "NV",
    "new hampshire": "NH",
    "new jersey": "NJ",
    "new mexico": "NM",
    "new york": "NY",
    "north carolina": "NC",
    "north dakota": "ND",
    "northern mariana islands": "MP",
    "ohio": "OH",
    "oklahoma": "OK",
    "oregon": "OR",
    "palau": "PW",
    "pennsylvania": "PA",
    "puerto rico": "PR",
    "rhode island": "RI",
    "south carolina": "SC",
    "south dakota": "SD",
    "tennessee": "TN",
    "texas": "TX",
    "utah": "UT",
    "vermont": "VT",
    "virgin islands": "VI",
    "virginia": "VA",
    "washington": "WA",
    "west virginia": "WV",
    "wisconsin": "WI",
    "wyoming": "WY",
}

NORMALIZE_MAP = {
    "prefix": DIRECTIONAL,
    "prefix1": DIRECTIONAL,
    "prefix2": DIRECTIONAL,
    "suffix": DIRECTIONAL,
    "suffix1": DIRECTIONAL,
    "suffix2": DIRECTIONAL,
    "type": STREET_TYPE,
    "type1": STREET_TYPE,
    "type2": STREET_TYPE,
    "state": STATE_CODE,
}



import re

from .config import NORMALIZE_MAP, DIRECTION_CODE
from .patterns import ADDR_MATCH
from .utils import is_number


def _normalize_address(parts=None):
    if parts is None:
        return

    parsed = {}

    valid_keys = (
        x for x in parts.keys() if not (x in ("input", "index") or is_number(x))
    )

    for key in valid_keys:
        new_key = (
            "_".join(key.split("_")[:-1]) if is_number(key.split("_").pop()) else key
        )
        if parts.get(key) is not None:
            parsed[new_key] = re.sub(r"^\s+|\s+$|[^\w\s\-#&]", "", parts[key].strip())

    for key, map_ in NORMALIZE_MAP.items():
        try:
            parsed[key] = map_[parsed[key].lower()]
        except KeyError:
            continue

    for key in ("type", "type1", "type2"):
        try:
            parsed[key] = parsed[key].capitalize()
        except KeyError:
            continue

    if parsed.get('city') is not None:
        parsed["city"] = re.sub(
            r"^(?P<dircode>{dircode})\s+(?=\S)".format(**ADDR_MATCH),
            lambda x: DIRECTION_CODE[x["dircode"].upper()].capitalize() + " ",
            parsed['city'],
            flags=re.I | re.X,
        )

    return parsed


def _parse_po_address(address):
    match = ADDR_MATCH['po_address'].search(address)
    return match and _normalize_address(match.groupdict())


def parse_address(address):
    match = ADDR_MATCH['address'].search(address)
    return match and _normalize_address(match.groupdict())


def parse_informal_address(address):
    match = ADDR_MATCH['informal_address'].search(address)
    return match and _normalize_address(match.groupdict())


def parse_intersection(address):
    match = ADDR_MATCH['intersection'].search(address)
    parts = match and _normalize_address(match.groupdict())

    if parts:
        parts['type2'] = parts.get('type2') or ''
        parts['type1'] = parts.get('type1') or ''
        if parts['type2'] and not parts['type1'] or (parts['type1'] == parts['type2']):
            type_ = re.sub(r's\W*$', '', parts['type2'])
            if re.search(r'^{}$'.format(ADDR_MATCH['type']), type_, re.X | re.I):
                parts['type1'] = parts['type2'] = type_

    return parts


def parse_location(address):
    if re.search(ADDR_MATCH['corner'], address, re.X | re.I):
        return parse_intersection(address)

    if re.search(r'^' + ADDR_MATCH['po_box'], address, re.X | re.I):
        return _parse_po_address(address)

    return parse_address(address) or parse_informal_address(address)



import re
from itertools import chain

from .config import DIRECTIONAL, STREET_TYPE, STATE_CODE, DIRECTION_CODE

SEP = r"(?:\W+|$)"

ADDR_MATCH = {
    "type": "|".join(sorted(set(chain(STREET_TYPE.keys(), STREET_TYPE.values())))),
    "fraction": r"\d+\/\d+",
    "state": r"\b(?:{})\b".format(
        "|".join([re.escape(x) for x in chain(STATE_CODE.keys(), STATE_CODE.values())])
    ),
    "direct": "|".join(
        list(DIRECTIONAL.keys())
        + list(
            chain.from_iterable(
                [
                    (re.escape(".".join(x) + "."), x)
                    for x in reversed(
                        sorted(DIRECTIONAL.values(), key=lambda x: len(x))
                    )
                ]
            )
        )
    ),
    "dircode": "|".join(DIRECTION_CODE.keys()),
    "zip": r"(?P<zip>\d{5})[- ]?(?P<plus4>\d{4})?",
    "corner": r"(?:\band\b|\bat\b|&|\@)",
    "number": r"(?P<number>(\d+-?\d*)|([N|S|E|W]\d{1,3}[N|S|E|W]\d{1,6}))(?=\D)",
    "po_box": r"p\W*(?:[om]|ost\ ?office)\W*b(?:ox)?",
    "sec_unit_type_unnumbered": r"""
(?P<sec_unit_type_2>ba?se?me?n?t
    |fro?nt
    |lo?bby
    |lowe?r
    |off?i?ce?
    |pe?n?t?ho?u?s?e?
    |rear
    |side
    |uppe?r
)\b
""",
}

ADDR_MATCH[
    "street"
] = r"""
(?:
    (?:(?P<street_0>{direct})\W+
        (?P<type_0>{type})\b
    )
    |
    (?:(?P<prefix_0>{direct})\W+)?
    (?:
        (?P<street_1>[^,]*\d)
        (?:[^\w,]*(?P<suffix_1>{direct})\b)
        |
        (?P<street_2>[^,]+)
        (?:[^\w,]+(?P<type_2>{type})\b)
        (?:[^\w,]+(?P<suffix_2>{direct})\b)?
        |
        (?P<street_3>[^,]+?)
        (?:[^\w,]+(?P<type_3>{type})\b)?
        (?:[^\w,]+(?P<suffix_3>{direct})\b)?
    )
)
""".format(
    **ADDR_MATCH
)

ADDR_MATCH[
    "sec_unit_type_numbered"
] = r"""
(?P<sec_unit_type_1>su?i?te
    |{po_box}
    |(?:ap|dep)(?:ar)?t(?:me?nt)?
    |ro*m
    |flo*r?
    |uni?t
    |bu?i?ldi?n?g
    |ha?nga?r
    |lo?t
    |pier
    |slip
    |spa?ce?
    |stop
    |tra?i?le?r
    |box)(?![a-z]
)
""".format(
    **ADDR_MATCH
)

ADDR_MATCH[
    "sec_unit"
] = r"""
(?:
    (?:
        (?:
            (?:{sec_unit_type_numbered}\W*)
            |(?P<sec_unit_type_3>\#)\W*
        )
        (?P<sec_unit_num_1>[\w-]+)
    )
    |
    {sec_unit_type_unnumbered}
)
""".format(
    **ADDR_MATCH
)

ADDR_MATCH[
    "city_and_state"
] = r"""
(?:
    (?P<city>[^\d,]+?)\W+
    (?P<state>{state})
)
""".format(
    **ADDR_MATCH
)

ADDR_MATCH[
    "place"
] = r"""
(?:{city_and_state}\W*)?
(?:{zip})?
""".format(
    **ADDR_MATCH
)

ADDR_MATCH["address"] = re.compile(
    r"""
^
[^\w#]*
({number})\W*
(?:{fraction}\W*)?
    {street}\W+
(?:{sec_unit})?\W*
    {place}
\W*$
""".format(
        **ADDR_MATCH
    ),
    re.I | re.X,
)

ADDR_MATCH["informal_address"] = re.compile(
    r"""
^
\s*
(?:{sec_unit_plus_sep})?
(?:{number})?\W*
(?:{fraction}\W*)?
    {street_plus_sep}
(?:{sec_unit_replaced})?
(?:{place})?
""".format(
        sec_unit_plus_sep=ADDR_MATCH["sec_unit"] + SEP,
        street_plus_sep=ADDR_MATCH["street"] + SEP,
        sec_unit_replaced=re.sub(r"_\d", r"\g<0>1", ADDR_MATCH["sec_unit"]) + SEP,
        **ADDR_MATCH
    ),
    re.I | re.X,
)

ADDR_MATCH["po_address"] = re.compile(
    r"""
^
\s*
(?:{sec_unit_replaced})?
(?:{place})?
""".format(
        sec_unit_replaced=re.sub(r"_\d", r"\g<0>1", ADDR_MATCH["sec_unit"]) + SEP,
        **ADDR_MATCH
    ),
    re.I | re.X,
)

ADDR_MATCH["intersection"] = re.compile(
    r"""
^\W*
{street_replaced_1}\W*?
\s+{corner}\s+
{street_replaced_2}\W+
{place}\W*$
""".format(
        street_replaced_1=re.sub(r"_\d", r"1\g<0>", ADDR_MATCH["street"]),
        street_replaced_2=re.sub(r"_\d", r"2\g<0>", ADDR_MATCH["street"]),
        **ADDR_MATCH
    ),
    re.I | re.X,
)



from setuptools import setup

import addresser

setup(
    name="addresser",
    description="A Python port of the `parse-address` npm library",
    long_description="A Python port of the `parse-address` npm library",
    long_description_content_type="text/x-rst",
    keywords="address parser normalizer usa us parse-address",
    version=addresser.__version__,
    packages=["addresser"],
    tests_require=["pytest"],
    python_requires=">=3.4",
    url="https://github.com/consbio/addresser",
    license="MIT",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)



import math
import numbers


def is_number(x):
    try:
        if x == math.inf or math.isnan(x):
            return False
    except TypeError:  # raised if x is not a number or NaN/inf
        pass

    if isinstance(x, numbers.Number):
        return True

    try:
        float(x)
    except ValueError:
        return False
    else:
        return True
